import os
import re
import logging
from twisted.internet import reactor

from mmc.support.mmctools import shLaunchDeferred
from mmc.plugins.packages.backend.base import Status


logger = logging.getLogger(__name__)


class UrpmiBackend(object):

    def __init__(self, refresh, need_reboot):
        self._status = Status.IDLE
        self._output = ""
        self.updates = []
        self.refresh = refresh
        self.need_reboot = need_reboot

    def is_available(self):
        return os.path.exists('/etc/urpmi/urpmi.cfg')

    def list(self, filter=""):
        if filter:
            updates = []
            for update in self.updates:
                if any(filter in s for s in update):
                    updates.append(update)
            return updates
        return self.updates

    def check(self):
        def store_updates_list(r, backend):
            if r.exitCode == 0:
                backend.updates = []
                for line in r.out.split("\n"):
                    m = re.match("([^ ]*)\s+:\s+([^(]*)\s+\(\s+([^)]*)\s+\)$", line)
                    if m:
                        backend.updates.append((m.group(1), m.group(2), m.group(3)))
                logger.debug("%s updates found." % len(backend.updates))
                if len(backend.updates) > 0:
                    self._status = Status.HAS_UPDATES
                else:
                    self._status = Status.IDLE
            else:
                logger.error("Failed to list new updates (%s)" % r.exitCode)
                self._status = Status.CHECKING_FAILED
            self._output = r.stdall

        def get_updates_list(r, backend):
            if r.exitCode == 0:
                logger.debug("Getting updates list...")
                d = shLaunchDeferred("urpmq --auto-select -S")
                # d = shLaunchDeferred("test_updates.py")
                d.addCallback(store_updates_list, backend)
            else:
                logger.error("Failed to check new updates (%s)" % r.exitCode)
                logger.error(r.err)
                backend._status = Status.CHECKING_FAILED
            self._output = r.stdall
            reactor.callLater(self.refresh, backend.check)
        if self._status not in (Status.CHECKING, Status.REBOOT):
            logger.debug("Check for updates...")
            self._status = Status.CHECKING
            d = shLaunchDeferred("urpmi.update -a")
            d.addCallback(get_updates_list, self)

    def run(self):
        def update_done(r, backend):
            if r.exitCode == 0:
                logger.debug("Update done")
                backend._status = Status.IDLE
                for p in self.need_reboot:
                    if re.search(p, r.stdall, re.MULTILINE):
                        backend._status = Status.REBOOT
                        break
            else:
                logger.debug("Failed to update packages (%s)" % r.exitCode)
                logger.error(r.err)
                backend._status = Status.UPDATING_FAILED
            # store command output
            backend._output = r.stdall
            # refresh updates
            backend.check()
        if self._status is not Status.UPDATING:
            logger.debug("Updating system packages...")
            self._status = Status.UPDATING
            d = shLaunchDeferred("urpmi --auto-select --auto")
            # d = shLaunchDeferred("test_updates.py")
            d.addCallback(update_done, self)

    @property
    def status(self):
        return (self._status, self._output.replace(">>", "").replace("<<", ""))
